#.onLoad <- function(libpath,package){
#    library.dynam.load("AdequateBootstrap",libpath)
#}
#
.onUnload <- function(package,libpath){
    packagefilename<-attr(packageDescription("AdequateBootstrap"),"file")
    library.dynam.unload("AdequateBootstrap",libpath=sub("/Meta.*","",packagefilename))
}


Checkdata<-function(X){
    if(!is.list(X)&!is.numeric(X)){
        base::error("X must be numeric.")        
    }

    if(is.list(X)){
        if(length(X)>3){
            base::error("Data can be either a vector, a vector with frequencies, or an interval with frequencies")
        }
        frequency=NULL
        if(length(X)==3){
            if(!is.numeric(X[[1]])||!is.numeric(X[[2]])){
                base::error("Interval boundaries should be numeric.")
            }
            frequency=X[[3]]
            X[[3]]=as.integer(X[[3]])
        }
        if(length(X)==2){
            if(!is.numeric(X[[1]])){
                base::error("X values should be numeric.")
            }
            frequency=X[[2]]
            X[[2]]=as.integer(X[[2]])
        }
        if(!is.numeric(frequency)){
            if(!all(frequency==as.integer(frequency))){
                base::error("Frequency should be an integer.")
            }
        }
        
    }
}


adequatebootstrap<-function(X,distribution,test){
##Check arguments
##X should be a vector
##distribution should name a distribution

###Check X is either a vector or a list of real vectors

    Checkdata(X)
    
###Check distribution is a named list of strings
    if(!is.list(distribution)&!is.character(distribution)){
        base::stop("Name of distribution should be a string.")        
    }

    if(is.list(distribution)){
        for(i in 1:length(distribution)){
            if(!is.character(distribution[[i]])){
                if(is.numeric(distribution[[i]])){
                    distribution[[i]]<-as.character(distribution[[i]])
                }else{
                    base::stop("Distribution parameter should be a string.",sep="")
                }
            }
        }
    }else{
        distribution<-list(distribution)
    }

    
###Check test is a string
    if(!is.character(test)){
        base::stop("Name of test should be a string.")
    };
    
##Call C function
    answer<-vector("double",4)
    returnval<-.Call('adequatebootstrap',X,distribution,test)
    names(returnval)<-c("size","interval","Distribution","test")
    returnval$Distribution<-list("Name"=returnval$Distribution[[1]][[1]],"MainParameter"=returnval$Distribution[[1]][[2]],"ParameterValues"=returnval$Distribution[[2]])
    names(returnval$test)<-c("name","sl")
    class(returnval)<-"abInterval"
    return(returnval)
}

    
adequatebootstrap_discriminant<-function(X1,X2,distribution1,distribution2,test,testdata){

###Check X is either a vector or a list of real vectors
    Checkdata(X1)
    Checkdata(X2)
    Checkdata(testdata)
    
###Check distribution is a named list of strings
    if(!is.list(distribution1)&!is.character(distribution1)){
        base::stop("Name of distribution 1 should be a string.")        
    }

    if(is.list(distribution1)){
        for(i in 1:length(distribution1)){
            if(!is.character(distribution1[[i]])){
                base::stop("Distribution 1 parameter should be a string.")
            }
        }
    }


    if(!is.list(distribution2)&!is.character(distribution2)){
        base::stop("Name of distribution 2 should be a string.")        
    }

    if(is.list(distribution2)){
        for(i in 1:length(distribution2)){
            if(!is.character(distribution2[[i]])){
                base::stop("Distribution 2 parameter should be a string.")
            }
        }
    }

    
###Check test is a string
    if(!is.character(test)){
        base::stop("Name of test should be a string.")
    };


###Call C function
    answer<-vector("double",4)
    returnval<-.Call('adequatebootstrap_discriminant',X1,X2,testdata,distribution1,distribution2,test)
    return(returnval)
}


print.abInterval<-function(x,...){
    cat(paste(x$Distribution$Name," distribution.\n\n",sep=""))
    cat("Fixed parameters:\n")
    for(i in 1:length(x$Distribution$ParameterValues)){
        cat(paste(names(x$Distribution$ParameterValues)[i],"=",x$Distribution$ParameterValues[i],"\n",sep=""))
    }
    cat(paste("\n",x$test$name," test at ",x$test$sl," significance level.\n\n",sep=""))
    cat(paste("Adequate bootstrap size = ",x$size,".\n\n",sep=""))    
    cat(paste("Adequate bootstrap interval for ",x$Distribution$MainParameter," is [",x$interval[1],",",x$interval[2],"].\n",sep=""))    
}
