//Simulation stuff.

#include "data.h"
#include "datatypes.h"
#include "distributions.h"
#include "specific_distributions.h"
#include <boost/math/distributions/normal.hpp>



using namespace std;

extern boost::math::normal stdnorm;


inline long double pnorm(long double x){
  return erfc((-x)/sqrt(2))/2;
}

inline long double qnorm(long double p){
  return quantile(stdnorm,p);
}

inline long double rnorm(){
  return quantile(stdnorm,((long double)rand())/RAND_MAX);
}



Data* NormalDistribution::simulate(unsigned int n) const{
  long double *values=new long double[n];
  for(unsigned int i=0;i<n;i++){
    *(values+i)=rnorm()*sigma+mu;
  };
  Data* answer=new Continuous_Individual_Data(n,values);
  answer->sort();
  return answer;
};  

Data* ParetoType1Distribution::simulate(unsigned int n) const{
  long double *values=new long double[n];
  long double ninval=-1/alpha;
  for(unsigned int i=0;i<n;i++){
    long double r=((long double)rand())/RAND_MAX;
    *(values+i)=min*pow(r,ninval);
  };
  Data* answer=new Continuous_Individual_Data(n,values);
  answer->sort();
  return answer;
};  



Data* PoissonDistribution::simulate(unsigned int n) const{
  unsigned int *values=new unsigned int[n];
  for(unsigned int i=0;i<n;i++){
    long double runif=((long double)rand())/RAND_MAX;
    *(values+i)=this->quant(runif);
  };
  Data *answer=new Discrete_Individual_Data(n,values);
  answer->sort();
  return answer;
};  


Data* LogNormalPoissonDistribution::simulate(unsigned int n) const{
  //Might get better numerical stability for tails
  //by counting backwards.
  //Need to figure out how to implement this.
  unsigned int *values=new unsigned int[n];
  for(unsigned int i=0;i<n;i++){
    long double lambda=exp(rnorm()*sigma+mu);
    long double runif=((long double)rand())/RAND_MAX;
    long double target=runif*exp(lambda);;
    long double total=1;
    unsigned int j=0;
    long double current=1;
    for(;total<target;j++){
      current*=lambda/j;
      total+=current;
    };
    *(values+i)=j;
  };
  Data *answer=new Discrete_Individual_Data(n,values);
  answer->sort();
  return answer;
};  

  
