#include "ParetoDistribution.h"
#include <cmath>

using namespace std;

void getpos(unsigned int n,const long double *list,unsigned int numps,const unsigned int *pos,long double *to);

unsigned int ParetoType1Distribution::numestpars() const{
  unsigned int ans=2;
  if(fixedalpha){
    ans--;
  };
  if(fixedmin){
    ans--;
  };
  return ans;
};


long double* ParetoType1Distribution::dist(const Data&dat) const{
  //No checking on suitability of dat. Should be checked by program.
  const long double *data=(const long double*)dat.getvalues();
  long double *answer=new long double[dat.n];
  for(unsigned int i=0;i<dat.n;i++){
    if(*(data+i)>min){
      *(answer+i)=1-pow(min/(*(data+i)),alpha);
    }else{
      *(answer+i)=0;
    };
  };
  return answer;
};

long double* ParetoType1Distribution::likelihood(const Data&dat) const{
  //No checking on suitability of dat. Should be checked by program.
  const long double *data=(const long double*)dat.getvalues();
  long double *answer=new long double[dat.n];
  for(unsigned int i=0;i<dat.n;i++){
    if(*(data+i)>min){
      *(answer+i)=alpha*pow(min/(*(data+i)),alpha+1)/min;
    }else{
      *(answer+i)=0;
    };
  };
  return answer;
};

long double* ParetoType1Distribution::survive(const Data&dat) const{
  //No checking on suitability of dat. Should be checked by program.
  const long double *data=(const long double *)dat.getvalues();
  long double *answer=new long double[dat.n];
  for(unsigned int i=0;i<dat.n;i++){
    if(*(data+i)>min){
      *(answer+i)=pow(min/(*(data+i)),alpha);
    }else{
      *(answer+i)=1;
    };
  };
  return answer;
};


long double ParetoType1Distribution::quant(long double p) const{
  return min*pow(1-p,-1/alpha);
};

long double ParetoType1Distribution::parval() const{
  switch(mainpar){
  case 0:
    return alpha;
  case 1:
    return min;
  default: //could check for corruption.
    return this->quant(VaR_at);
  };
};

long double ParetoType1Distribution::parval(unsigned int i,long double at) const{
  switch(i){
  case 0:
    return alpha;
  case 1:
    return min;
  default:
    return this->quant(at);
  };
};

#define CRITVALSAMPLESIZE 1000

//long double ParetoType1Distribution::adjustADstat(unsigned int samplesize) const{};

ParetoType1Distribution* ParetoType1Distribution::copy() const{
  return new ParetoType1Distribution(*this);
};

void ParetoType1Distribution::setfixed(unsigned int i,long double val){
  switch(i){
  case 0:
    alpha=val;
    fixedalpha=1;
    break;
  case 1:
    min=val;
    fixedmin=1;
    break;
  default:
    throw Distribution_error("Fixing VaR not currently possible for Pareto distribution.\n");
  };
};


void ParetoType1Distribution::setfixed(unsigned int i){
  switch(i){
  case 0:
    fixedalpha=1;
    break;
  case 1:
    fixedmin=1;
    break;
  default:
    throw Distribution_error("Fixing VaR not currently possible for Pareto distribution.\n");
  };
};


void ParetoType1Distribution::unfix(unsigned int i){
  switch(i){
  case 0:
    fixedalpha=0;
    break;
  case 1:
    fixedmin=0;
    break;
  default:
    break;
    //VaR can't be fixed anyway.
    //Perhaps should give some indication of a problem.
  };
};

void ParetoType1Distribution::setmain(unsigned int i,long double at){
  mainpar=i;
  if(i==2){
    VaR_at=at;
  };
};

void ParetoType1Distribution::estimate(const Data &dat){
  const long double *data=(const long double *)dat.getvalues();
  if(fixedmin){
    for(unsigned int i=0;i<dat.n;i++){
      if(*(data+i)<min){
	throw Distribution_error("Data point below minimum specified value for Type I Pareto distribution.\nRerun with new minimum specified\n");
      };
    };
  }else{
    if(*data<=0){
      throw Distribution_error("Only positive values are possible for the Pareto distribution.\n");
    };
    min=*(data);
    for(unsigned int i=1;i<dat.n;i++){
      if(*(data+i)<min){
	if(*(data+i)<=0){
	  throw Distribution_error("Only positive values are possible for the Pareto distribution.\n");
	};
	min=*(data+i);
      };
    };    
  };
  if(!fixedalpha){
    long double slx=0;
    unsigned int x=0;
    if(dat.get_type().storage&(LIST|FREQUENCY_TABLE)){
      const unsigned int *freq=(const unsigned int *)dat.getfreqs();
      if(freq){
	for(unsigned int i=0;i<dat.n;i++){
	  if(*(data+i)>min){//truncate at min.
	    slx+=log(*(data+i));
	    x++;
	  };
	};
      }else{
	for(unsigned int i=0;i<dat.n;i++){
	  if(*(data+i)>min){//truncate at min.
	    slx+=log(*(data+i))*(*(freq+i));
	    x+=(*(freq+i));
	  };
	};
      };    
      slx-=x*log(min);
      alpha=x/slx;	
    }else{
      throw Distribution_error("Pareto Type 1 Distribution parameter estimation not currently available for data stored in this format.\n");
    };
  };
};

