#include "TDistribution.h"
#include <cmath>
#include <boost/math/special_functions/beta.hpp>

using namespace std;

void getpos(unsigned int n,const long double *list,unsigned int numps,const unsigned int *pos,long double *to);

unsigned int TDistribution::numestpars() const{
  unsigned int ans=3;
  if(fixedmu){
    ans--;
  };
  if(fixedsigma){
    ans--;
  };
  if(fixeddegreesoffreedom){
    ans--;
  };
  return ans;
};


long double TDistribution::parval() const{
  switch(mainpar){
  case 0:
    return mu;
  case 1:
    return sigma;
  case 2:
    return degreesoffreedom;
  default: //could check for corruption.
    return this->quant(VaR_at);
  };
};

long double TDistribution::parval(unsigned int i,long double at) const{
  switch(i){
  case 0:
    return mu;
  case 1:
    return sigma;
  case 2:
    return degreesoffreedom;
  default:
    return this->quant(at);
  };
};


TDistribution* TDistribution::copy() const{
  return new TDistribution(*this);
};

void TDistribution::setfixed(unsigned int i,long double val){
  switch(i){
  case 0:
    mu=val;
    fixedmu=1;
    break;
  case 1:
    sigma=val;
    fixedsigma=1;
    break;
  case 2:
    degreesoffreedom=val;
    fixeddegreesoffreedom=1;
    break;
  default:
    throw Distribution_error("Fixing VaR not currently possible for t distribution.\n");
  };
};

void TDistribution::setfixed(unsigned int i){
  switch(i){
  case 0:
    fixedmu=1;
    break;
  case 1:
    fixedsigma=1;
    break;
  case 2:
    fixeddegreesoffreedom=1;
    break;
  default:
    throw Distribution_error("Fixing VaR not currently possible for t distribution.\n");
  };
};


void TDistribution::unfix(unsigned int i){
  switch(i){
  case 0:
    fixedmu=0;
    break;
  case 1:
    fixedsigma=0;
    break;
  case 2:
    fixeddegreesoffreedom=0;
    break;
  default:
    break;
    //VaR can't be fixed anyway.
    //Perhaps should give some indication of a problem.
  };
};

void TDistribution::setmain(unsigned int i,long double at){
  mainpar=i;
  if(i==3){
    VaR_at=at;
  };
};









long double* TDistribution::dist(const Data&dat) const{
  //No checking on suitability of dat. Should be checked by program.
  const long double *data=(const long double*)dat.getvalues();
  long double *answer=new long double[dat.n];
  for(unsigned int i=0;i<dat.n;i++){
    long double t=(*(data+i)-mu)/sigma;
    long double x=degreesoffreedom/(t*t+degreesoffreedom);
    *(answer+i)=1-boost::math::ibeta(degreesoffreedom/2, 0.5, x)/2;
  };
  return answer;
};

long double* TDistribution::likelihood(const Data&dat) const{
  //No checking on suitability of dat. Should be checked by program.
  const long double *data=(const long double*)dat.getvalues();
  long double *answer=new long double[dat.n];
  long double constant=exp(lgamma((degreesoffreedom+1)/2)-lgamma(degreesoffreedom/2))/(sqrt(3.1415926535897962643383*degreesoffreedom)*sigma);
  for(unsigned int i=0;i<dat.n;i++){
    long double t=(*(data+i)-mu)/sigma;    
    *(answer+i)=constant*pow(1+t*t/degreesoffreedom,-(degreesoffreedom+1)/2);
  };
  return answer;
};

long double* TDistribution::survive(const Data&dat) const{
  //No checking on suitability of dat. Should be checked by program.
  const long double *data=(const long double *)dat.getvalues();
  long double *answer=new long double[dat.n];
  for(unsigned int i=0;i<dat.n;i++){
    long double t=(*(data+i)-mu)/sigma;
    long double x=degreesoffreedom/(t*t+degreesoffreedom);
    *(answer+i)=boost::math::ibeta(degreesoffreedom/2, 0.5, x)/2;
  };
  return answer;
};


long double TDistribution::quant(long double p) const{
  throw Distribution_error("Quantile function not yet implemented for t distribution.\n");
};


void TDistribution::estimate(const Data &dat){
  throw Distribution_error("Parameter estimation not yet implemented for t distribution.\n");
  const long double *data=(const long double *)dat.getvalues();
  if(fixedsigma){
  }else{
  };
  if(!fixedmu){
    if(dat.get_type().storage&(LIST|FREQUENCY_TABLE)){
      const unsigned int *freq=(const unsigned int *)dat.getfreqs();
      if(freq){
      }else{
      };    
    }else{
      throw Distribution_error("t Distribution parameter estimation not currently available for data stored in this format.\n");
    };
  };
};

