/*

   Copyright 2016 Toby Kenney

   This file is part of Adequate Bootstrap.

   Adequate Bootstrap is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Adequate Bootstrap is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Adequate Bootstrap.  If not, see <http://www.gnu.org/licenses/>.

 */


#include "distributions.h"
#include "data.h"
#include "corefunctions.h"
#include <iostream>
#include <fstream>
#include <exception>

/*
#include <cmath>
#include <cstdlib>
#include <ctime>


#include <boost/math/distributions/normal.hpp>
#include <boost/math/distributions/chi_squared.hpp>
#include <boost/math/distributions/binomial.hpp>
#include <boost/random/mersenne_twister.hpp>
#include <boost/random/variate_generator.hpp>
#include <boost/generator_iterator.hpp>
#include <boost/random/binomial_distribution.hpp>
#include <boost/random/mersenne_twister.hpp>
#include <boost/random/normal_distribution.hpp>
*/

#define BSSIZE 10000
//Number of bootstraps to perform.


#include <R.h>
#include <Rcpp.h>
#include <Rinternals.h>


using namespace std;
using namespace Rcpp;
//using namespace Rinternals;


#define NUMBOOTSTRAP 10000


//3 modes of operation:
//1st mode: data analysis


SEXP discriminant_analysis(Distribution &dist1,Distribution &dist2,AdequacyTest &adequacy_test,Data &cla1,Data &cla2,Data &test){
  //Fits dist1 to class 1, and dist2 to class 2, using adequate
  //bootstrap Calculates the likelihood of test under each class, and
  //thus gets the probability of datapoints in test being in each class.
  //  cout<<"Performing discriminant analysis.\n";
  unsigned int mx=(cla1.n>cla2.n)?cla1.n:cla2.n;
  mx-=-19;
  if(cla1.n<20||cla2.n<20){
    error("Adequate Bootstrap cannot currently deal with datasets of size less than 20.\n");
  };
  unsigned int *xvals=new unsigned int[mx];
  for(unsigned int i=0;i<mx;i++){
    *(xvals+i)=i+20;
  };

  //  cout<<"Class sizes: "<<cla1.n<<"\t"<<cla2.n<<"\n";
  unsigned int c1bssz=solveprobfast(cla1.n-19,xvals,adequacy_test,dist1,0.5,cla1);
  unsigned int c2bssz=solveprobfast(cla2.n-19,xvals,adequacy_test,dist2,0.5,cla2);

  //  cout<<"Adequate Bootstrap size:\n Class 1: "<<c1bssz<<"\nClass 2: "<<c2bssz<<"\n\n";
  
  long double *probs=new long double[test.n*BSSIZE];
  long double *stdbsprobs=new long double[test.n*BSSIZE];
  //  long double *probs2=new long double[test2.n*BSSIZE];
  for(unsigned int i=0;i<BSSIZE;i++){
    cla1.subsample(c1bssz);
    dist1.estimate(cla1);
    cla2.subsample(c2bssz);
    dist2.estimate(cla2);
    long double *lik1=dist1.likelihood(test);
    long double *lik2=dist2.likelihood(test);
    cla1.subsample(cla1.n);
    dist1.estimate(cla1);
    cla2.subsample(cla2.n);
    dist2.estimate(cla2);
        long double *lik3=dist1.likelihood(test);
        long double *lik4=dist2.likelihood(test);
    for(unsigned int j=0;j<test.n;j++){
      *(probs+j*BSSIZE+i)=*(lik1+j)/(*(lik1+j)+*(lik2+j));
      *(stdbsprobs+j*BSSIZE+i)=*(lik3+j)/(*(lik3+j)+*(lik4+j));
    };
    //    for(unsigned int j=0;j<test2.n;j++){
    //      *(probs2+j*BSSIZE+i)=*(lik3+j)/(*(lik3+j)+*(lik4+j));
    //    };
    delete[] lik1;
    delete[] lik2;
        delete[] lik3;
        delete[] lik4;
  };
  long double *CI=new long double[2*test.n];
  unsigned int pos[2]={(unsigned int)(0.025*BSSIZE),(unsigned int)(0.975*BSSIZE)};
  SEXP answer = PROTECT(Rf_allocVector(VECSXP, 5));
  SEXP ablower = PROTECT(Rf_allocVector(REALSXP, test.n));
  SEXP abupper = PROTECT(Rf_allocVector(REALSXP, test.n));
  SEXP sblower = PROTECT(Rf_allocVector(REALSXP, test.n));
  SEXP sbupper = PROTECT(Rf_allocVector(REALSXP, test.n));
  double *abl=REAL(ablower);
  double *abu=REAL(abupper);
  double *sbl=REAL(sblower);
  double *sbu=REAL(sbupper);
  for(unsigned int i=0;i<test.n;i++){
    getpos(BSSIZE,probs+i*BSSIZE,2,pos,CI+2*i);
    *(abl+i)=*(CI+2*i);
    *(abu+i)=*(CI+2*i+1);
  };
  SEXP absizes = PROTECT(Rf_allocVector(REALSXP, 2));
  REAL(absizes)[0]=c1bssz;
  REAL(absizes)[1]=c2bssz;
  SET_VECTOR_ELT(answer,0,absizes);
  //  cout<<"\nAdequate Bootstrap:\n";
  //  cout<<"\nStandard Bootstrap:\n";
  long double *CIstd=new long double[2*test.n];
  for(unsigned int i=0;i<test.n;i++){
    getpos(BSSIZE,stdbsprobs+i*BSSIZE,2,pos,CIstd+2*i);
    *(sbl+i)=*(CIstd+2*i);
    *(sbu+i)=*(CIstd+2*i+1);
  };
  SET_VECTOR_ELT(answer,1,ablower);
  SET_VECTOR_ELT(answer,2,abupper);
  SET_VECTOR_ELT(answer,3,sblower);
  SET_VECTOR_ELT(answer,4,sbupper);
  UNPROTECT(5);
  return(answer);
};



AdequateBootstrapIntervals  analyse_data(Distribution &truedist,AdequacyTest &adequacy_test,Data &A){
  //Analyse Data
  //  truedist.print();

  if(A.n<20){
    error("Adequate Bootstrap cannot currently deal with datasets of size less than 20.\n");
  };

  truedist.estimate(A);
  
  unsigned int *xvals=new unsigned int[A.n-19];
  for(unsigned int i=0;i<A.n-19;i++){
    *(xvals+i)=i+20;
  };
  unsigned int bssize=solveprobfast(A.n-19,xvals,adequacy_test,truedist,0.5,A);

  delete[] xvals;

  AdequateBootstrapIntervals ans;

  ans.absz=bssize;


  long double *bs=new long double[BSSIZE];
  long double *stdbs=new long double[BSSIZE];

  long double *absamp=new long double[bssize];
  long double *bsamp=new long double[A.n];
  unsigned int *sbsmp=new unsigned int[A.n];

  for(unsigned int i=0;i<BSSIZE;i++){
    A.subsample(bssize);
    truedist.estimate(A);
    *(bs+i)=truedist.parval();
  };
  
  for(unsigned int i=0;i<BSSIZE;i++){
    A.subsample(A.n);
    truedist.estimate(A);
    *(stdbs+i)=truedist.parval();
  };

  unsigned int pos[2]={(unsigned int)(0.025*BSSIZE),(unsigned int)(0.975*BSSIZE)};

  long double ci[2];
  long double stdci[2];

  //  long double confprob[2]={0.025*exp(target),0.975*exp(target)};


  getpos(BSSIZE,bs,2,pos,ci);
  getpos(BSSIZE,stdbs,2,pos,stdci);

  
  ans.abl=ci[0];
  ans.abu=ci[1];
  ans.sbl=stdci[0];
  ans.sbu=stdci[1];


  //Initialise to avoid unnecessary compiler warnings.
  ans.nbl=0;
  ans.nbu=0;
  ans.nl=0;
  ans.nu=0;


  delete[] absamp;
  delete[] bsamp;
  delete[] sbsmp;
  delete[] bs;
  delete[] stdbs;

  return ans;
};


long double chisqlookup(unsigned int n,long double atval);

extern long double testatval;

//
//void Discriminant_Analysis_mode(){
//  //currently performs discriminant analysis between two classes.
//  //Should extend to multiclass analysis.
//  Distribution *dist1;
//  Distribution *dist2;
//  AdequacyTest *adequacy_test;
//  istream *datafile1;
//  istream *datafile2;  
//  istream *testdatafile;  
//  try{
//    dist1=(Distribution*)lookup_option("distribution");
//    if(dist1==NULL){
//      cout<<"Distribution for analysis not specified";
//      exit(0);
//    };
//    dist1->print();
//    dist2=(Distribution*)lookup_option("distribution2");
//    if(dist2==NULL){
//      cout<<"Distribution for second class not specified. Using same distribution for both classes.";
//      dist2=dist1->copy();//Default is to use same distribution for
//    };
//    dist2->print();
//
//    //Next read adequacy test  (mandatory argument)
//    adequacy_test=(AdequacyTest*)lookup_option("adequacy_test");
//    if(adequacy_test==NULL){	
//      cout<<"Adequacy test not specified.\n";
//      exit(0);
//    };                    
//    datafile1=(istream*)lookup_option("file");
//    datafile2=(istream*)lookup_option("file2");
//    testdatafile=(istream*)lookup_option("testdata");
//    if(datafile1==NULL&&datafile2==NULL){
//      cout<<"No data files specified. Cannot read both from standard input.\n";
//    };
//    if(datafile1==NULL){
//      cout<<"No data file specified. Reading data from standard input.\n";
//      datafile1=&cin;
//    };
//    if(datafile2==NULL){
//      cout<<"No data file specified. Reading data from standard input.\n";
//      datafile2=&cin;
//    };
//  }catch(exception& e){
//    cout<<e.what()<<"\n";
//    exit(0);
//  };
//  datatype type__data(dist1->type_of_data(),dataformat());
//  datatype type__data2(dist1->type_of_data(),dataformat());
//  Data *data1=new_data(type__data);
//  data1->read(*datafile1);
//  Data *data2=new_data(type__data2);
//  data2->read(*datafile2);
//  Data *testdata;
//  if(testdatafile==NULL){
//    cout<<"No test data file specified. Concatenating input file.\n";
//    testdata=data1->copy();
//    testdata->concat(data2);
//  }else{
//    testdata=new_data(type__data);
//    testdata->read(*testdatafile);
//  };
//  discriminant_analysis(*dist1,*dist2,*adequacy_test,*data1,*data2,*testdata);
//};
//
//
//
//void Analysis_mode(){
//  Distribution *truedist;
//  AdequacyTest *adequacy_test;
//  istream *datafile;
//  try{
//    truedist=(Distribution*)lookup_option("distribution");
//    if(truedist==NULL){
//	cout<<"Distribution for analysis not specified";
//    };
//    truedist->print();
//    //Next read adequacy test  (mandatory argument)
//    adequacy_test=(AdequacyTest*)lookup_option("adequacy_test");
//    if(adequacy_test==NULL){	
//      cout<<"Adequacy test not specified.\n";
//      exit(0);
//    };
//    if(*(unsigned int*)lookup_option("estimateparams")){
//      adequacy_test->estimateparameters();
//    };    
//    datafile=(istream*)lookup_option("file");
//    if(datafile==NULL){
//      cout<<"No data file specified. Reading data from standard input.\n";
//      datafile=&cin;
//    };
//  }catch(exception& e){
//    cout<<e.what()<<"\n";
//    exit(0);
//  };
//  datatype type__data(truedist->type_of_data(),dataformat());
//  Data *data=new_data(type__data);
//  data->read(*datafile);
//  analyse_data(*truedist,*adequacy_test,*data);
//};
//




//void test_mode(int argc,char *argv[]);


Data* makedata(SEXP X);

SEXP getSEXP(distSummary a){
  //Reads a linked list of information into a SEXP
  //  Rcout<<a.name<<"\n";
  //  Rcout<<a.mainpar<<"\n";
  //  Rcout<<a.numpar<<"\n";

  
  Rcpp::List answer(2);//=PROTECT(Rf_allocVector(VECSXP, a.numpar+2));
  //  SET_STR_ELT(distname,0, a.name);
  //STRSEXP distname(a.name,a.mainpar);
  //  SEXP distname = PROTECT(Rf_allocVector(CHARSXP, 1));
  //  SEXP distname = PROTECT(CHARSXP(a.name));


  SEXP distname = PROTECT(Rf_allocVector(STRSXP, 2));
  
  SET_STRING_ELT(distname,0,Rf_mkChar(a.name));
   SET_STRING_ELT(distname,1,Rf_mkChar(a.mainpar));


   //   CharacterVector distnames=CharacterVector::create("Distribution","Main parameter")
  
  //    distname[0]=(std::string)a.name;
    //  Rcpp::StringVector distmainpar[1];
  //  distname[1]=(std::string)a.mainpar;
  //  SEXP distmainpar = PROTECT(Rf_allocVector(STRSXP, 1));
   //  distname.names()=CharacterVector::create("Distribution","Main parameter")
  SET_VECTOR_ELT(answer,0,distname);
  //  answer[1]=(SEXP)distmainpar;
  //  SET_VECTOR_ELT(answer,0,distname);
  //  SET_VECTOR_ELT(answer,1,distmainpar);
  if(a.numpar){
    SEXP value = PROTECT(Rf_allocVector(REALSXP, a.numpar));
    for(unsigned int i=0;i<a.numpar;i++){
      REAL(value)[i]=*(a.paramvals+i);
    }
    SEXP parnms = PROTECT(Rf_allocVector(STRSXP, a.numpar));
    for(unsigned int i=0;i<a.numpar;i++){
      SET_STRING_ELT(parnms, i, Rf_mkChar(*(a.paramnames+i)));
    };
    Rf_setAttrib(value, R_NamesSymbol, parnms);
    answer[1]=value;
    UNPROTECT(2);
  };
  UNPROTECT(1);
  //  UNPROTECT(a.numpar);
  answer.names()=CharacterVector::create("name","Parameters");
  return answer;
};


extern "C"{
// [[Rcpp::export]]
  SEXP adequatebootstrap(SEXP X, SEXP distribution, SEXP adequacytest){
    //
    //    Rcpp::Rcout<<"Got here.\n";
    unsigned int n=Rf_length(X);
    //lookup distribution
    unsigned int dargc=Rf_length(distribution);
    char **distargs=new char *[dargc];
    SEXP distributionname=VECTOR_ELT(distribution,0);
    SEXP distributionnameCHARSXP=STRING_ELT(distributionname,0);
    //    Rcpp::Rcout<<"Got here.\n";
    const char *distname=CHAR(distributionnameCHARSXP);
    *distargs=new char[strlen(distname)+1];
    strcpy(*distargs,distname);
    SEXP names= Rf_getAttrib(distribution, R_NamesSymbol);
    for(unsigned int i=1;i<dargc;i++){
      SEXP argvalue=VECTOR_ELT(distribution,i);      
      const char* val=CHAR(STRING_ELT(argvalue,0));
      const char* name=CHAR(STRING_ELT(names,i));
      unsigned int len=strlen(name)+strlen(val);
      if(strcmp(name,"main")){
	unsigned int len=strlen(name)+strlen(val);
	*(distargs+i)=new char[len+3];
	strcpy(*(distargs+i),name);
	strcat(*(distargs+i),"=");
	strcat(*(distargs+i),val);
      }else{
	*(distargs+i)=new char[strlen(val)+1];
	strcpy(*(distargs+i),val);	
      };
      //      Rcpp::Rcout<<*(distargs+i)<<"\n";
    };

    unsigned int start=0;
    Distribution *dist=read_distribution(dargc,distargs,start);
    //    dist->print();
    //lookup adequacy_test
    const char *adtest=CHAR(STRING_ELT(adequacytest,0));
    //    Rcpp::Rcout<<"Got here.\n";
    AdequacyTest *test=read_adequacy_test(adtest);
    Data *data=makedata(X);
    AdequateBootstrapIntervals  ans=analyse_data(*dist,*test,*data);
    SEXP interval = PROTECT(Rf_allocVector(REALSXP, 2));
    REAL(interval)[0]=ans.abl;
    REAL(interval)[1]=ans.abu;
    SEXP answer = PROTECT(Rf_allocVector(VECSXP, 4));
    SEXP adbootsize = PROTECT(Rf_allocVector(REALSXP, 1));    
    REAL(adbootsize)[0]=ans.absz;
    SEXP distdetails = getSEXP(dist->Rsummary());

    SEXP testdetails = PROTECT(Rf_allocVector(VECSXP, 2));

    SEXP testname = PROTECT(Rf_allocVector(STRSXP, 1));
    SET_STRING_ELT(testname,0,Rf_mkChar(test->testname()));
    SEXP testsize = PROTECT(Rf_allocVector(REALSXP, 1));    
    REAL(testsize)[0]=test->get_level();

    
    SET_VECTOR_ELT(testdetails,0,testname);
    SET_VECTOR_ELT(testdetails,1,testsize);

    
    SET_VECTOR_ELT(answer,0,adbootsize);
    SET_VECTOR_ELT(answer,1,interval);
    SET_VECTOR_ELT(answer,2,distdetails);
    SET_VECTOR_ELT(answer,3,testdetails);
    UNPROTECT(6);
    return answer;
  };

  SEXP adequatebootstrap_discriminant(SEXP X1,SEXP X2, SEXP testdata, SEXP distribution1, SEXP distribution2, SEXP adequacytest){
    //Technically, we could use different adequacy tests for the same classes, but It doesn't seem there is much reason for doing so.   
    unsigned int n=Rf_length(X1);
    //lookup distribution
    unsigned int dargc=Rf_length(distribution1);
    unsigned int dargc2=Rf_length(distribution2);
    char **distargs=new char *[dargc];
    char **distargs2=new char *[dargc2];
    SEXP distributionname=VECTOR_ELT(distribution1,0);
    SEXP distributionnameCHARSXP=STRING_ELT(distributionname,0);
    //    Rcpp::Rcout<<"Got here.\n";
    const char *distname=CHAR(distributionnameCHARSXP);
    SEXP distributionname2=VECTOR_ELT(distribution2,0);
    SEXP distributionnameCHARSXP2=STRING_ELT(distributionname2,0);
    //    Rcpp::Rcout<<"Got here.\n";
    const char *distname2=CHAR(distributionnameCHARSXP2);
    *distargs=new char[strlen(distname)+1];
    strcpy(*distargs,distname);
    SEXP names= Rf_getAttrib(distribution1, R_NamesSymbol);
    for(unsigned int i=1;i<dargc;i++){
      SEXP argvalue=VECTOR_ELT(distribution1,i);      
      const char* val=CHAR(STRING_ELT(argvalue,0));
      //      Rcpp::Rcout<<"Got here."<<i<<"\n";
      const char* name=CHAR(STRING_ELT(names,i));
      unsigned int len=strlen(name)+strlen(val);
      *(distargs+i)=new char[len+3];
      strcpy(*(distargs+i),name);
      strcat(*(distargs+i),"%=");
      strcat(*(distargs+i),val);
    };

    *distargs2=new char[strlen(distname2)+1];
    strcpy(*distargs2,distname2);
    names= Rf_getAttrib(distribution2, R_NamesSymbol);
    for(unsigned int i=1;i<dargc2;i++){
      SEXP argvalue=VECTOR_ELT(distribution2,i);      
      const char* val=CHAR(STRING_ELT(argvalue,0));
      const char* name=CHAR(STRING_ELT(names,i));
      if(strcmp(name,"main")){
	unsigned int len=strlen(name)+strlen(val);
	*(distargs2+i)=new char[len+3];
	strcpy(*(distargs2+i),name);
	strcat(*(distargs2+i),"%=");
	strcat(*(distargs2+i),val);
      }else{
	*(distargs2+i)=new char[strlen(name)+1];
	strcpy(*(distargs2+i),name);	
      };
    };

    
    unsigned int start=0;
    Distribution *dist=read_distribution(dargc,distargs,start);
    start=0;
    Distribution *dist2=read_distribution(dargc2,distargs2,start);
    //lookup adequacy_test
    const char *adtest=CHAR(STRING_ELT(adequacytest,0));
    AdequacyTest *test=read_adequacy_test(adtest);
    Data *data=makedata(X1);
    Data *data2=makedata(X2);
    Data *testdat=makedata(testdata);
    SEXP answer=discriminant_analysis(*dist,*dist2,*test,*data,*data2,*testdat);
    UNPROTECT(1);
    return answer;
  };

  

}
