#include "specific_distributions.h"
#include <cstring>
#include <exception>

//For debugging. Remove this line when debugging finished
//#include <Rcpp.h>
//#include <iostream>

class DistributionlistElement{
  const char* name;
  Distribution &dist;
  DistributionlistElement *next;
  DistributionlistElement();
public:
  DistributionlistElement(const char *nm,Distribution &dst);
  unsigned int totalnamelength();
  void listallnames(char *ans);
  void insert(const char *nm,Distribution &dst);
  Distribution* match(const char *nm);
};

Distribution* DistributionlistElement::match(const char *nm){
  if(!strcmp(name,nm)){
    return dist.copy();//Need to make a copy so can use same
			//distribution for multiple variables.
  }else if(next==NULL){
    return NULL;
  }else{
    return next->match(nm);
  };
};


DistributionlistElement::DistributionlistElement(const char *nm,Distribution &dst):name(nm),dist(dst),next(NULL){
};

void DistributionlistElement::insert(const char *nm,Distribution &dst){
  DistributionlistElement*nx=next;
  next=new DistributionlistElement(nm,dst);
  next->next=nx;
};

unsigned int DistributionlistElement::totalnamelength(){
  if(next){
    return strlen(name)+3+next->totalnamelength();
  }else{
    return strlen(name)+1;
  };
};

void DistributionlistElement::listallnames(char *ans){
  unsigned int len=strlen(name);
  *ans='\"';
  strcpy(ans+1,name);
  *(ans+len+1)='\"';
  *(ans+len+2)=' ';
  if(next){
    next->listallnames(ans+len+3);
  }else{
    *(ans+len+3)='\0';
  };
};



DistributionlistElement *distributions=NULL;

void set_up_distributions(){
   distributions=new DistributionlistElement("normal",*(new NormalDistribution()));
  //  distributions->insert("log_normal",*(new LogNormalDistribution()));
    distributions->insert("pareto",*(new ParetoType1Distribution()));
    distributions->insert("Poisson",*(new PoissonDistribution()));
    distributions->insert("log_normal_Poisson",*(new LogNormalPoissonDistribution()));
    distributions->insert("t",*(new TDistribution()));
};


char *listofpermitteddistributions(){
  if(distributions==NULL){
    set_up_distributions();
  };
  char *ans=new char[distributions->totalnamelength()];
  distributions->listallnames(ans);
  return ans;
};

Distribution* get_distribution(const char *const name){
  if(distributions==NULL){
    set_up_distributions();
  };
  return distributions->match(name);
};


class distribution_formatting_error: public std::exception{
  static const char*const header;
  char * message;
public:
  distribution_formatting_error(const char *const mess){message=new char[strlen(mess)+1];strcpy(message,mess);};
  distribution_formatting_error(const char *const mess,const char*values,const char *end){message=new char[strlen(mess)+strlen(values)+strlen(end)+1];strcpy(message,mess);strcat(message,values);strcat(message,end);};
  virtual const char* what() const throw() {char *full_message=new char[strlen(header)+strlen(message)+1];strcpy(full_message,header);strcat(full_message,message);return full_message;}; 
};

const char*const distribution_formatting_error::header="Distribution formatting error: ";


Distribution* read_distribution(int argc,const char * const argv[],unsigned int &start){
  Distribution* orig=get_distribution(argv[start]);
  if(orig==NULL){
    //could not find matching distibution
    throw distribution_formatting_error("Distribution \"",argv[start],"\" not known\n");
  };
  Distribution *ans=orig->copy();//Not freed at exit
  for(start++;start<(unsigned)argc;start++){
    //    Rcpp::Rcout<<argv[start]<<"\n";
    char *at=NULL;
    char *eq=NULL;
    unsigned int percent=0;
    char* name=new char[strlen(argv[start])+1];
    strcpy(name,argv[start]);
    unsigned int len=strlen(name);
    for(unsigned int i=0;i<len;i++){
      if(*(name+i)=='@'){
	*(name+i)='\0';
	at=(name+i+1);
      }else if(*(name+i)=='%'){
	*(name+i)='\0';
	percent=1;
      }else if(*(name+i)=='='){
	*(name+i)='\0';
	eq=(name+i+1);	
      };
    };
    long double val=0;
    if(eq!=NULL){
      val=atof(eq);
    };
    long double atval=0;
    if(at!=NULL){
      atval=atof(at);
      if(eq!=NULL&&eq<at){
	*eq='\0';
	*(eq-1)='@';
      }else{
	*at='\0';
	*(at-1)='@';
      };
    };
    unsigned int i=0;
    for(;i<ans->numvalidpars&&strcmp(name,*(ans->validparnames()+i));i++);
    if(i==ans->numvalidpars){
      delete[] name;
      return ans;
    };
    if(eq!=NULL){
      ans->setfixed(i,val);
      //      cout<<"set parameter "<<i<<" to value "<<val<<".\n";
      //      ans->print();
      if(percent){
	ans->unfix(i);
      };
    }else{
      ans->setmain(i,atval);
    };   
    delete[] name;
  };
  return ans;
};
