//#include "tests.h"
#include "distributions.h"

#include <iostream>
#include <fstream>
#include <cmath>
#include <boost/math/distributions/normal.hpp>
#include <boost/math/distributions/binomial.hpp>
#include <boost/math/distributions/non_central_f.hpp>
#include <boost/math/distributions/fisher_f.hpp>
#include <boost/random/mersenne_twister.hpp>
#include <boost/random/variate_generator.hpp>
#include <boost/generator_iterator.hpp>
#include <boost/random/binomial_distribution.hpp>
#include <boost/random/mersenne_twister.hpp>
#include <boost/random/normal_distribution.hpp>

#include <cstdlib>
#include <ctime>

using namespace std;


class AndersonDarling: public AdequacyTest{
private:
  long double critval;
public:
  AndersonDarling():AdequacyTest(datatype(CONTINUOUS_DATA|DISCRETE_DATA,LIST|FREQUENCY_TABLE|GROUPED)),critval(0){};
  AndersonDarling(long double cv):AdequacyTest(datatype(CONTINUOUS_DATA|DISCRETE_DATA,LIST|FREQUENCY_TABLE|GROUPED)),critval(cv){};
  virtual char apply(const Data& dat,const Distribution &dist);  
  virtual long double statistic(const Data& dat,const Distribution &dist);
  virtual void set_level(long double size,const Distribution &dist){critval=dist.get_critval(size,*this);};  
  using AdequacyTest::set_level;  
  using AdequacyTest::get_level;  
  virtual const char* testname(){return "Anderson-Darling";};
};

char AndersonDarling::apply(const Data& dat,const Distribution &dist){
  //use lazy calculation of critval.
  if(critval==0){
    critval=dist.get_critval(level,*this);
  };
  return this->statistic(dat,dist)>critval;
};  

long double AndersonDarling::statistic(const Data &dat,const Distribution &dist){
  Distribution *tempdist=dist.copy();
  if(estimateparams){
    tempdist->estimate(dat);
  };

  long double *distfuns=tempdist->dist(dat);
  
  long double ADstat=-1;
  unsigned int Fn=0;
  long double Fsp=0;
  long double Fsn=0;
  //    long double Fsnc=0;
  const unsigned int *freq=dat.getfreqs();
  unsigned int tot=dat.n;
  if(freq){
    tot=0;
    for(unsigned int i=0;i<dat.n;i++){
      tot+=*(freq+i);
    };
  };
  for(unsigned int j=0;j<dat.n;j++){
    Fsp=distfuns[j];
    Fsn=1-Fsp;
    if(Fsn==0&&(freq==NULL||*(freq+j))){
      //Too implausible.
      return critval*100;
      //reject
    };
      
      
    ADstat-=(1-((long double)Fn)/dat.n)*(1-((long double)Fn)/dat.n)*log(Fsn);
    ADstat+=(((long double)Fn)/dat.n)*(((long double)Fn)/dat.n)*log(Fsp);
    if(freq){
      Fn+=*(freq+j);
    }else{
      Fn++;
    };
    ADstat+=(1-((long double)Fn)/dat.n)*(1-((long double)Fn)/dat.n)*log(Fsn);
    ADstat-=(((long double)Fn)/dat.n)*(((long double)Fn)/dat.n)*log(Fsp);
  };
  long double ans=ADstat*dat.n;//*tempdist->adjustADstat(n);
  delete[] distfuns;
  delete tempdist;
  return ans;
};



//Kolmogorov-Smirnov test


class KolmogorovSmirnov: public AdequacyTest{
private:
  long double critval;
public:
  KolmogorovSmirnov():AdequacyTest(datatype(CONTINUOUS_DATA|DISCRETE_DATA,LIST|FREQUENCY_TABLE|GROUPED)),critval(0){};
  KolmogorovSmirnov(long double cv):AdequacyTest(datatype(CONTINUOUS_DATA|DISCRETE_DATA,LIST|FREQUENCY_TABLE|GROUPED)),critval(cv){};
  virtual char apply(const Data& dat,const Distribution &dist);  
  virtual long double statistic(const Data& dat,const Distribution &dist);
  virtual void set_level(long double size,const Distribution &dist){critval=dist.get_critval(size,*this);};  
  using AdequacyTest::set_level;  
  using AdequacyTest::get_level;  
  virtual const char* testname(){return "Kolmogorov-Smirnov";};
  //  virtual void set_level(long double cv){critval=cv;};  
};

char KolmogorovSmirnov::apply(const Data &dat , const Distribution &dist){
  if(critval==0){
    critval=dist.get_critval(level,*this);
  };
  return this->statistic(dat,dist)>critval;  
};

long double KolmogorovSmirnov::statistic(const Data &dat , const Distribution &dist){
  Distribution *tempdist=dist.copy();
  if(estimateparams){
    tempdist->estimate(dat);
  };
  long double KSstat=0;
  unsigned int Fn=0;
  long double Fsp=0;
  long double *dists=tempdist->dist(dat);
  const unsigned int *freq=dat.getfreqs();
  unsigned int tot=dat.n;
  if(freq){
    tot=0;
    for(unsigned int i=0;i<dat.n;i++){
      tot+=*(freq+i);
    };
  };
  for(unsigned int j=0;j<dat.n;j++){
    Fsp=tot*(*(dists+j));
    if(Fsp-Fn>KSstat){
      KSstat=Fsp-Fn;
    };
    if(freq){
      Fn+=*(freq+j);
    }else{
      Fn++;
    };
    if(Fn-Fsp>KSstat){
      KSstat=Fn-Fsp;
    };
  };
  delete tempdist;
  delete[] dists;
  return KSstat/sqrt(tot);
};



class ChiSquare: public AdequacyTest{
private:
  long double critval;
public:
  ChiSquare():AdequacyTest(datatype(CONTINUOUS_DATA|DISCRETE_DATA,LIST|FREQUENCY_TABLE|GROUPED)),critval(0){};
  ChiSquare(long double cv):AdequacyTest(datatype(CONTINUOUS_DATA|DISCRETE_DATA,LIST|FREQUENCY_TABLE|GROUPED)),critval(cv){};
  virtual char apply(const Data& dat,const Distribution &dist);  
  virtual long double statistic(const Data& dat,const Distribution &dist);
  virtual void set_level(long double size,const Distribution &dist){critval=dist.get_critval(size,*this);};  
  using AdequacyTest::set_level;  
  using AdequacyTest::get_level;  
  virtual const char* testname(){return "Pearson's chi-square";};
  //  virtual void set_level(long double cv){critval=cv;};  
};


char ChiSquare::apply(const Data &dat , const Distribution &dist){
  if(critval==0){
    critval=dist.get_critval(level,*this);
  };
  return this->statistic(dat,dist)>critval;  
};

long double ChiSquare::statistic(const Data &dat,const Distribution &dist){
  Distribution *tempdist=dist.copy();
  if(estimateparams){
    tempdist->estimate(dat);
  };
  long double ans=0;
  long double ldist=0;

  long double *dists=tempdist->dist(dat);
  const unsigned int *samp=dat.getfreqs();
  unsigned int tot=dat.n;
  if(samp){
    tot=0;
    for(unsigned int i=0;i<dat.n;i++){
      tot+=*(samp+i);
    };
  };
  
  for(unsigned int i=0;i<dat.n;i++){
    unsigned int bincount=0;
    for(;i<dat.n&&(*(dists+i)-ldist)*tot<20;i++){
      if(samp){
	bincount+=*(samp+i);
      }else{
	bincount++;
      };
    };
    long double dst=1;
    if(i<dat.n-1){
      dst=*(dists+i);
    };
    ans+=(bincount-(dst-ldist)*tot)*(bincount-(dst-ldist)*tot)/((dst-ldist)*tot);
    ldist=dst;
  };
  delete tempdist;
  delete[] dists;
  return ans;  
};




long double chisqlookup(unsigned int n,long double atval);


//*/

AdequacyTest* read_adequacy_test(const char*  testname){
  char *temp=new char[strlen(testname)+1];
  strcpy(temp,testname);
  char *at=NULL;
  unsigned int i=0;
  for(;*(temp+i)!='\0'&&*(temp+i)!='@';i++);
  if(*(temp+i)=='@'){
    at=temp+i+1;
    *(temp+i)='\0';
  };
  long double testatval=0.05;
  if(!strcmp(temp,"AD")){//Anderson-Darling test    
    if(at!=NULL){
      testatval=atof(at);
    };
    AndersonDarling *AD=new AndersonDarling();
    AD->set_level(testatval);
    delete[] temp;
    return AD;
  }else if(!strcmp(temp,"KS")){//Kolmogorov-Smirnov test
    if(at!=NULL){
      testatval=atof(at);
    };
    KolmogorovSmirnov *KS=new KolmogorovSmirnov();
    KS->set_level(testatval);
    delete[] temp;
    return KS;
  }else if(!strcmp(temp,"ChiSquare")){//Pearson's Chi-square test
    if(at!=NULL){
      testatval=atof(at);
    };
    delete[] temp;
    ChiSquare *CS=new ChiSquare();
    CS->set_level(testatval);
    return CS;
  }else{
    delete[] temp;
    return NULL;
  };
};

