//#include <iostream>
#include "NormalDistribution.h"
#include <cmath>
#include <boost/math/distributions/normal.hpp>

extern boost::math::normal stdnorm;
using namespace std;

inline long double pnorm(long double x){
  return erfc((-x)/sqrt(2))/2;
}

inline long double qnorm(long double p){
  return quantile(stdnorm,p);
}

inline long double rnorm(){
  return quantile(stdnorm,((long double)rand())/RAND_MAX);
}

void getpos(unsigned int n,const long double *list,unsigned int numps,const unsigned int *pos,long double *to);

unsigned int NormalDistribution::numestpars() const{
  unsigned int ans=2;
  if(fixedmu){
    ans--;
  };
  if(fixedsig){
    ans--;
  };
  return ans;
};


long double* NormalDistribution::dist(const Data&dat) const{
  //No checking on suitability of dat. Should be checked by program.
  const long double *values=(const long double*)dat.getvalues();
  long double *answer=new long double[dat.n];
  for(unsigned int i=0;i<dat.n;i++){
    *(answer+i)=pnorm((*(values+i)-mu)/sigma);
  };
  return answer;
};


long double* NormalDistribution::likelihood(const Data&dat) const{
  //No checking on suitability of dat. Should be checked by program.
  const long double *values=(const long double*)dat.getvalues();
  long double *answer=new long double[dat.n];
  for(unsigned int i=0;i<dat.n;i++){
    *(answer+i)=exp(-(*(values+i)-mu)*(*(values+i)-mu)/(2*sigma*sigma))/(sqrt(2*3.141592653589793238462643383)*sigma);
  };
  return answer;
};


long double* NormalDistribution::survive(const Data&dat) const{
  //No checking on suitability of dat. Should be checked by program.
  const long double *values=(const long double *)dat.getvalues();
  long double *answer=new long double[dat.n];
  for(unsigned int i=0;i<dat.n;i++){
    *(answer+i)=1-pnorm((*(values+i)-mu)/sigma);
  };
  return answer;
};


long double NormalDistribution::quant(long double p) const{
  return qnorm(p)*sigma+mu;
};

long double NormalDistribution::parval() const{
  switch(mainpar){
  case 0:
    return mu;
  case 1:
    return sigma;
  default: //could check for corruption.
    return this->quant(VaR_at);
  };
};

long double NormalDistribution::parval(unsigned int i,long double at) const{
  switch(i){
  case 0:
    return mu;
  case 1:
    return sigma;
  default:
    return this->quant(at);
  };
};

#define CRITVALSAMPLESIZE 1000

long double Distribution::get_critval(long double size,AdequacyTest& test)  const{
  long double adval[10000];
  for(unsigned int i=0;i<10000;i++){
    Data *data=this->simulate(CRITVALSAMPLESIZE);
    adval[i]=test.statistic(*data,*this);
    delete data;
  };
  unsigned int pos=(long double)10000*(1-size);
  if(pos>9999){
    pos=9999;
  };
  long double ans=0;
  getpos(10000,adval,1,&pos,&ans);
  return(ans);  
};

unsigned int Distribution::setpar(const char *na,const char *val){
  unsigned int len=strlen(na);
  char *parname=new char[len+1];
  strcpy(parname,na);
  unsigned int percent=0;
  if(len&&*(parname+len-1)=='%'){
    (*(parname+len-1))='\0';
    percent=1;
  };
  char *at=NULL;
  for(unsigned int j=0;*(parname+j);j++){
    if(*(parname+j)=='@'){
      *(parname+j)='\0';
      at=(parname+j+1);
    };
  };
  unsigned int i=0;
  const char *const* vlprs=this->validparnames();
  for(;i<numvalidpars&&strcmp(parname,*(vlprs+i));i++);
  if(i==numvalidpars){
    delete[] parname;
    return 0;
  }else{
    if(val){
      this->setfixed(i,atof(val));
      if(percent){
	this->unfix(i);
      };
    }else{
      this->setmain(i,atof(at));
    };
    delete[] parname;
    return 1;
  };
};


//long double NormalDistribution::adjustADstat(unsigned int samplesize) const{};

NormalDistribution* NormalDistribution::copy() const{
  return new NormalDistribution(*this);
};

void NormalDistribution::setfixed(unsigned int i,long double val){
  switch(i){
  case 0:
    mu=val;
    fixedmu=1;
    break;
  case 1:
    sigma=val;
    fixedsig=1;
    break;
  default:
    throw Distribution_error("Fixing VaR not currently possible for normal distribution.\n");
  };
};

void NormalDistribution::unfix(unsigned int i){
  switch(i){
  case 0:
    fixedmu=0;
    break;
  case 1:
    fixedsig=0;
    break;
  default:
    break;
    //VaR can't be fixed anyway.
    //Perhaps should give some indication of a problem.
  };
};

void NormalDistribution::setmain(unsigned int i,long double at){
  mainpar=i;
  if(i==2){
    VaR_at=at;
  };
};

void NormalDistribution::estimate(const Data &dat){
  long double sum=0;
  long double ssq=0;
  unsigned int n;
  if(dat.get_type().storage&(LIST|FREQUENCY_TABLE)){
    const long double *data=(const long double *)dat.getvalues();
    const unsigned int *freq=(const unsigned int *)dat.getfreqs();
    n=dat.n;
    if(freq){
      n=0;
      for(unsigned int i=0;i<dat.n;i++){
	sum+=*(data+i)*(*(freq+i));
	ssq+=*(data+i)*(*(data+i))*(*(freq+i));      
	n+=*(freq+i);
      };    
    }else{
      for(unsigned int i=0;i<n;i++){
	sum+=*(data+i);
	ssq+=*(data+i)*(*(data+i));      
      };
    };
  }else{
    throw Distribution_error("Normal Distribution parameter estimation not currently available for data stored in this format.\n");
  };
  if(!fixedmu){
    mu=sum/n;
  };
  if(!fixedsig){
    sigma=sqrt(ssq/n+mu*mu-2*sum*mu/n);
  };
};


