#ifndef PARETODISTRIBUTION_H
#define PARETODISTRIBUTION_H

#include "data.h"
#include "distributions.h"

//ParetoType1 Distribution

#define SUPERCONSTRUCT Distribution("pareto",datatype(CONTINUOUS_DATA,LIST|FREQUENCY_TABLE|GROUPED),2,3)

const char *const paretovalidparnames[]={"alpha","min","VaR@"};

class ParetoType1Distribution: public Distribution{
protected:
  long double alpha;
  long double min;
  char fixedalpha;
  char fixedmin;
  unsigned int mainpar;
  long double VaR_at;
public:
  ParetoType1Distribution(const ParetoType1Distribution &orig):SUPERCONSTRUCT,alpha(orig.alpha),min(orig.min),fixedalpha(orig.fixedalpha),fixedmin(orig.fixedmin),mainpar(orig.mainpar),VaR_at(orig.VaR_at){};
  ParetoType1Distribution(long double m,long double s):SUPERCONSTRUCT,alpha(m),min(s),fixedalpha(0),fixedmin(0),mainpar(0),VaR_at(0){};
  ParetoType1Distribution():SUPERCONSTRUCT,alpha(0),min(1),fixedalpha(0),fixedmin(0),mainpar(0),VaR_at(0){};
  virtual ~ParetoType1Distribution(){};
  virtual unsigned int numpars() const{return 2;};
  virtual unsigned int numvalidpars() const{return 3;};
  virtual const char *const*validparnames() const{return paretovalidparnames;};
  virtual long double* dist(const Data&dat) const;
  virtual long double* survive(const Data&dat) const;
  virtual long double *likelihood(const Data &dat) const;
  virtual long double quant(long double p) const;
  virtual Data* simulate(unsigned int n) const;  
  virtual long double parval() const;
  virtual long double parval(unsigned int i,long double at) const;
  virtual void setfixed(unsigned int i,long double val);
  virtual void unfix(unsigned int i);
  virtual void setmain(unsigned int i,long double at);
  //  virtual long double get_critval(long double size,AdequacyTest& test)  const;
  //  virtual long double adjustADstat(unsigned int samplesize) const;
  virtual unsigned int numestpars() const;
  virtual ParetoType1Distribution* copy() const;
  virtual void estimate(const Data &dat);
  virtual void print() const;
};


#undef SUPERCONSTRUCT
#endif
