#ifndef POISSON_H
#define POISSON_H

#include "data.h"
#include "distributions.h"

#define SUPERCONSTRUCT Distribution("Poisson",datatype(DISCRETE_DATA,LIST|FREQUENCY_TABLE|GROUPED),1,1)

class PoissonDistribution: public Distribution{
protected:
  static const char* const valparnams[];
  unsigned int fixedlambda;
  long double lambda;
public:
 PoissonDistribution(const PoissonDistribution &orig):SUPERCONSTRUCT,fixedlambda(orig.fixedlambda),lambda(orig.lambda){};
 PoissonDistribution(long double lam):SUPERCONSTRUCT,fixedlambda(0),lambda(lam){};
 PoissonDistribution():SUPERCONSTRUCT,fixedlambda(0),lambda(1){};
  ~PoissonDistribution(){};
  virtual const char *const*validparnames() const{return valparnams;};
  virtual void setfixed(unsigned int i);
  virtual void setfixed(unsigned int i,long double val);
  virtual void unfix(unsigned int i){};
  virtual void setmain(unsigned int i,long double at){};
  virtual long double parval() const;
  virtual long double parval(unsigned int i,long double at) const;
  virtual unsigned int numestpars() const{return 1;};
  virtual PoissonDistribution* copy() const;
  virtual void estimate(const Data &dat);//for grouped data
  virtual Data * simulate(unsigned int n) const;  

  
  virtual long double *dist(const Data &dat) const;
  virtual long double *survive(const Data &dat) const;
  virtual long double *likelihood(const Data &dat) const;
  virtual long double quant(long double p) const;

  virtual void print() const;
};

#undef SUPERCONSTRUCT
#endif
