#ifndef DISTRIBUTIONS_H
#define DISTRIBUTIONS_H

#include "data.h"

class AdequacyTest;



class Distribution{
 protected:
  datatype type_data;
  unsigned int numpars;
  unsigned int numvalidpars;
  const char *distname;
 public:
 Distribution(const char *name,datatype tp,unsigned int np,unsigned int nvp):type_data(tp),numpars(np),numvalidpars(nvp),distname(name){};
  unsigned int compatible(const Data &dat){return dat.get_type().match(type_data);};
  virtual const char *const*validparnames() const=0;
  virtual void setfixed(unsigned int i,long double val)=0;
  virtual void unfix(unsigned int i)=0;
  virtual void setmain(unsigned int i,long double at)=0;
  virtual long double parval() const=0;
  virtual long double parval(unsigned int i,long double at) const=0;
  long double get_critval(long double size,AdequacyTest& teststat) const;
  virtual Distribution* copy() const=0;
  virtual void estimate(const Data &dat)=0;
  virtual unsigned int numestpars() const=0;
  virtual unsigned int numparameters(){return numpars;};
  virtual Data *simulate(unsigned int n) const=0;  
  unsigned int setpar(const char *na,const char *val);

  //virtual long double adjustADstat(unsigned int samplesize) const=0;

  virtual long double *dist(const Data &dat) const=0;
  virtual long double *survive(const Data &dat) const=0;
  virtual long double *likelihood(const Data &dat) const=0;
  virtual long double quant(long double p) const=0;

 
  virtual void print() const=0;
  virtual ~Distribution(){};


  friend Distribution* read_distribution(int argc,const char *const argv[],unsigned int &start);
  char type_of_data(){return type_data.type_of_data;};
 
  
  
};


Distribution* read_distribution(int argc,const char *const argv[],unsigned int &start);

class AdequacyTest{
 private:
  datatype data; //What types of data can this test be applied to
 protected:
  long double level;
 public:
 AdequacyTest(datatype dt):data(dt),level(0){};
  char compatible(const Data &d){return d.get_type().within(data);};
  virtual char apply(const Data& dat,const Distribution &dist)=0;
  virtual long double statistic(const Data& dat,const Distribution &dist)=0;
  virtual void set_level(long double size){level=size;};
};





AdequacyTest* read_adequacy_test(const char*  testname);

NPestimator& read_np_estimator(const char*  estimatorname,const Distribution &dist);


Distribution* get_distribution(const char *const name);

char *listofpermitteddistributions();

//class for returning errors in distribution handling
//should perhaps move this to special header only useful for distribution
//definitions.
#include <exception>

class Distribution_error: public std::exception{
  static const char*const header_string;
  char * message;
public:
  Distribution_error(const char *const mess);
  virtual const char* what() const throw(); 
};

//const char*const Distribution_error::header_string="Distribution error: ";


#endif
