/*

   Copyright 2016 Toby Kenney

   This file is part of Adequate Bootstrap.

   Adequate Bootstrap is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Adequate Bootstrap is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Adequate Bootstrap.  If not, see <http://www.gnu.org/licenses/>.

 */


#include "distributions.h"
#include "distortion.h"
//#include "tests.h"
#include "data.h"
#include "corefunctions.h"
#include <iostream>
#include <fstream>
#include <exception>

/*
#include <cmath>
#include <cstdlib>
#include <ctime>


#include <boost/math/distributions/normal.hpp>
#include <boost/math/distributions/chi_squared.hpp>
#include <boost/math/distributions/binomial.hpp>
#include <boost/random/mersenne_twister.hpp>
#include <boost/random/variate_generator.hpp>
#include <boost/generator_iterator.hpp>
#include <boost/random/binomial_distribution.hpp>
#include <boost/random/mersenne_twister.hpp>
#include <boost/random/normal_distribution.hpp>
*/

#define BSSIZE 10000
//Number of bootstraps to perform.




using namespace std;

extern long double chisqcval;


extern NPestimator *NPest;
extern NPestimator *CILest;
extern NPestimator *CIUest;



#define NUMBOOTSTRAP 10000


//3 modes of operation:
//1st mode: data analysis




void analyse_data(Distribution &truedist,AdequacyTest &adequacy_test,Data &A){
  //Analyse Data
  truedist.print();

  if(A.n<20){
    cout<<"Adequate Bootstrap cannot currently deal with datasets of size less than 20.\n";
    exit(0);
  };

  unsigned int *xvals=new unsigned int[A.n-19];
  for(unsigned int i=0;i<A.n-19;i++){
    *(xvals+i)=i+20;
  };
  unsigned int bssize=solveprobfast(A.n-19,xvals,adequacy_test,truedist,0.5,A);

  delete[] xvals;

  AdequateBootstrapIntervals ans;

  //  cout<<"Adequate Bootstrap size: "<<bssize<<".\n";
  ans.absz=bssize;


  long double *bs=new long double[BSSIZE];
  long double *stdbs=new long double[BSSIZE];
  long double *npbs=new long double[BSSIZE];

  long double *absamp=new long double[bssize];
  long double *bsamp=new long double[A.n];
  unsigned int *sbsmp=new unsigned int[A.n];

  for(unsigned int i=0;i<BSSIZE;i++){
    A.subsample(bssize);
    truedist.estimate(A);
    *(bs+i)=truedist.parval();
  };

  for(unsigned int i=0;i<BSSIZE;i++){
    A.subsample(A.n);
    truedist.estimate(A);
    *(stdbs+i)=truedist.parval();
    if(NPest!=NULL){
      *(npbs+i)=NPest->evaluate(A);
    };
  };

  unsigned int pos[2]={(unsigned int)(0.025*BSSIZE),(unsigned int)(0.975*BSSIZE)};

  long double ci[2];
  long double stdci[2];
  long double npci[2];

  //  long double confprob[2]={0.025*exp(target),0.975*exp(target)};

  //  cout<<target<<" "<<confprob[0]<<" "<<confprob[1]<<" ";

  getpos(BSSIZE,bs,2,pos,ci);
  getpos(BSSIZE,stdbs,2,pos,stdci);

  //  cout<<posl<<" "<<posu<<"\n";
  
  ans.abl=ci[0];
  ans.abu=ci[1];
  ans.sbl=stdci[0];
  ans.sbu=stdci[1];


  //Initialise to avoid unnecessary compiler warnings.
  ans.nbl=0;
  ans.nbu=0;
  ans.nl=0;
  ans.nu=0;


  if(NPest!=NULL){
    getpos(BSSIZE,npbs,2,pos,npci);
    ans.nbl=npci[0];
    ans.nbu=npci[1];
  };

  if(CILest!=NULL&&CIUest!=NULL){
    for(unsigned int i=0;i<A.n;i++){
      *(sbsmp+i)=1;
    };
    ans.nl=CILest->evaluate(A);
    ans.nu=CIUest->evaluate(A);
    //    cout<<cil_VaR_at<<" "<<ciu_VaR_at;
  };


  delete[] absamp;
  delete[] bsamp;
  delete[] sbsmp;
  delete[] bs;
  delete[] stdbs;
  delete[] npbs;


  cout<<"\nAdequate Bootstrap size: "<<ans.absz<<"\n";

  cout<<"Adequate Bootstrap interval: ["<<ans.abl<<", "<<ans.abu<<"]\n";
  cout<<"Standard Bootstrap interval: ["<<ans.sbl<<", "<<ans.sbu<<"]\n";

  if(NPest!=NULL){
    cout<<"Nonparametric Bootstrap interval: ["<<ans.nbl<<", "<<ans.nbu<<"]\n";
  };

  if(CILest!=NULL&&CIUest!=NULL){
    cout<<"Theoretical Confidence interval: ["<<ans.nl<<", "<<ans.nu<<"]\n";
  };  
};


long double chisqlookup(unsigned int n,long double atval);

extern long double testatval;

char dataformat;

void Analysis_mode(int argc,char *argv[]){
  //First try to identify distribution (mandatory arguments)
  unsigned int argno=2;

  Distribution *truedist=read_distribution(argc,argv,argno);

  //Next read adequacy test  (mandatory argument)
  if(truedist==NULL){	
    cout<<"Distribution \""<<argv[2]<<"\" not currently available.\n";
    cout<<"or not specified correctly.\n";
    exit(0);
  };
  AdequacyTest *adequacy_test=read_adequacy_test(argv[argno],*truedist);
  if(adequacy_test==NULL){	
    cout<<"Adequacy test \""<<argv[argno]<<"\" not currently available.\n";
    exit(0);
  };

  //Check to see if random seed specified.
  //Check for other options.
  for(argno++;argno<(unsigned int)argc;argno++){
    read_option(argv[argno]);
  };

  //Read data from file
  ifstream datafile(argv[1]);
  if(!datafile.is_open()){
    cerr<<"problem opening file \""<<argv[1]<<"\". Aborting.\n";
    exit(0);
  };

  datatype type__data(truedist->type_of_data(),dataformat);
  Data *data=new_data(type__data);
  data->read(datafile);
  analyse_data(*truedist,*adequacy_test,*data);
};


void Simulation_mode(int argc,char *argv[]){
  if(argc<4){
    cout<<"usage: \'"<<argv[0]<<" simulation SIMULATION_SIZE DISTRIBUTION PARAMETERS DISTORTION PARAMETERS ADEQUACY_TEST [OPTIONS]\n";
    cout<<"or: \'"<<argv[0]<<" test \'\n";
    exit(0);
  };
  Distribution *truedist;
  Distortion *distort;
  AdequacyTest *adequacy_test;
  Data *A;
  try{
    unsigned int simsize=atoi(argv[2]);
    unsigned int argno=3;
    truedist=read_distribution(argc,argv,argno);

    truedist->print();
    
    distort=Read_distortion(argno,argc,argv);
    
    if(distort==NULL){
      cout<<"Distotion type incorrectly specified.\n";
      exit(0);
    };
    
    //Next read adequacy test  (mandatory argument)
    if(argno>=(unsigned int)argc){
      cout<<"Adequacy test not specified.\n";
      exit(0);
    };
    adequacy_test=read_adequacy_test(argv[argno],*truedist);
    if(adequacy_test==NULL){	
      cout<<"Adequacy test \""<<argv[argno]<<"\" not currently available.\n";
      exit(0);
    };
    

  //Important to read options before simulating, since this allows us
  //to set the random number generator seed before the simulation.
    for(argno++;argno<(unsigned int)argc;argno++){
      read_option(argv[argno]);
    };
    
    
  
    A=distort->simulate(simsize,*truedist);

//    cout<<"\n";
//    A->write(cout,10);
//    cout<<"\n";
    
  }catch(exception& e){
    cout<<e.what()<<"\n";
    exit(0);
  };
  analyse_data(*truedist,*adequacy_test,*A);
};

//void test_mode(int argc,char *argv[]);


int main(int argc,char* argv[]){
  if(argc==1){
    cout<<"usage: \'"<<argv[0]<<" FILENAME DISTRIBUTION PARAMETERS ADEQUACY_TEST [OPTIONS]\n";
    cout<<"or: \'"<<argv[0]<<" simulation NUMBER DISTRIBUTION PARAMETERS DISTORTION PARAMETERS ADEQUACY_TEST [OPTIONS]\n";
    cout<<"or: \'"<<argv[0]<<" test \'\n";
    exit(0);
  };
//  if(!strcmp(argv[1],"test")){
//    test_mode(argc,argv);
//  }else
    if(!strcmp(argv[1],"simulation")){
    Simulation_mode(argc,argv);
  }else{    
    //data mode
    Analysis_mode(argc,argv);
  };
  cleanup();
};



