#ifndef DATA_H
#define DATA_H
#include <cstdlib>
#include <iosfwd>

class datatype{
  //Describes the different possible datatypes
  //Distributions and datasets should match exactly one datatype
  //Adequacy tests should match a range of possible datatypes;
 public:
  unsigned char type_of_data;
#define CONTINUOUS_DATA 1
#define DISCRETE_DATA 2
#define MULTIVARIATE_CONTINUOUS_DATA 4
#define MULTIVARIATE_DISCRETE_DATA 8
#define REGRESSION_DATA 16
  unsigned char  storage;
#define LIST 1
#define FREQUENCY_TABLE 2
#define GROUPED 4
 datatype(char ty,char st):type_of_data(ty),storage(st){};
 datatype():type_of_data(0),storage(0){};
  char match(const datatype &compare){return type_of_data==compare.type_of_data&&(storage==(storage&compare.storage));};//Compares for an exact match
  char within(const datatype &compare){return (type_of_data==(type_of_data&compare.type_of_data)&&(storage==(storage&compare.storage)));};//Compares for datatype covered
};



class Data{
 protected:
  unsigned int jackknife;
  datatype typ;
 public:
  unsigned int n;
  unsigned int *sample=NULL;
 Data(unsigned int m):jackknife(0),n(m){};
  datatype get_type() const{return typ;};
  virtual const void* getvalues() const=0;
  virtual const unsigned int* getfreqs()const=0;
  void setjackknife(unsigned int j=1){jackknife=j;};
  virtual void subsample(unsigned int size);
  virtual void clearsubsample(unsigned int size);//unlikely to be needed.
  virtual void read(std::istream &in)=0;
  virtual void concat(const Data *cdat)=0;
  virtual void sort()=0;
  virtual ~Data(){};
};


class NPestimator{
 public:
  datatype type; //What types of data can this test be applied to
  virtual long double evaluate(const Data& dat)=0;
 NPestimator(datatype t):type(t){};
 NPestimator():type(datatype(0,0)){};
  unsigned int compatible(const Data &dat){return dat.get_type().within(type);};
  ~NPestimator(){};
};


NPestimator *get_NP_estimator(const char *eq,const char *at);

Data* new_data(const datatype &type);


#endif
