
/*

   Copyright 2016 Toby Kenney

   This file is part of Adequate Bootstrap.

   Adequate Bootstrap is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Adequate Bootstrap is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Adequate Bootstrap.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "distributions.h"
#include "distortion.h"
#include <cstdlib>
#include <cstring>
#include <cstdio>
#include <exception>

unsigned int rbinom_fast(unsigned int n,long double p);

using namespace std;


class contamination: public Distortion{
  const Distribution *contaminating_distribution;
  long double probability;
public:
  contamination():contaminating_distribution(NULL),probability(0){};
  contamination(long double p,const Distribution *cd):contaminating_distribution(cd),probability(p){};
  void set_contamination(const Distribution *contam);
  void set_prob(long double pr);
  virtual Data *simulate(unsigned int simsize,const Distribution &truedist);
};

Data *contamination::simulate(unsigned int n,const Distribution &main){
  if(contaminating_distribution==NULL){
    return NULL;
  };
  unsigned int numcont=rbinom_fast(n,probability);  
  Data *dattrue=main.simulate(n-numcont);
  Data *datcontam=contaminating_distribution->simulate(numcont);
  dattrue->concat(datcontam);
  delete datcontam;
  return dattrue;
};



class no_distortion: public Distortion{
  virtual Data *simulate(unsigned int simsize,const Distribution &truedist);
};

Data *no_distortion::simulate(unsigned int simsize,const Distribution &truedist){
  return truedist.simulate(simsize);
};





class sample_bias_step: public Distortion{
unsigned int ncut;
long double *cuts;
long double *cum_probs;
public:
  sample_bias_step():ncut(0),cuts(NULL),cum_probs(NULL){};
  sample_bias_step(unsigned int nc,long double *cts,long double *prbs):ncut(0),cuts(cts),cum_probs(prbs){};
  virtual Data *simulate(unsigned int simsize,const Distribution &truedist);
};



Data *sample_bias_step::simulate(unsigned int n,const Distribution &main){
  //Need to fix this!
  //Doesn't work at all.

  if(cum_probs==NULL||cuts==NULL){
    return NULL;
  };
  long double *ans=new long double[n];
  //Assume probs sum to 1.
  unsigned int *nsim=new unsigned int[ncut+1];
  for(unsigned int i=0;i<=ncut;i++){
    *(nsim+i)=0;
  };
  for(unsigned int i=0;i<n;i++){
    long double rnd=((long double)rand())/RAND_MAX;
    unsigned int j=0;
    for(;j<ncut&&*(cum_probs+j)<rnd;j++);
    (*(nsim+j))++;
  };
  unsigned int *nsimdone=new unsigned int[ncut+1];
  for(unsigned int i=0;i<=ncut;i++){
    *(nsimdone+i)=0;
  };
  long double **bucket=new long double *[ncut+1];
  *bucket=ans;
  for(unsigned int i=0;i<ncut;i++){
    *(bucket+i+1)=*(bucket+i)+*(nsim+i);
  };
  unsigned int remaintosim=n;
  Data *dat=main.simulate(remaintosim);    
  while(remaintosim){
    for(unsigned int i=0;i<n;){
      unsigned int sim=0;
      unsigned int j=0;
      for(;j<ncut&&*(cuts+j)<sim;j++);    
      if(*(nsimdone+j)<*(nsim+j)){
	remaintosim--;
	i++;
	*(*(bucket+j)+(*(nsimdone+j))++)=sim;
      };
    };
  };
  
  delete[] bucket;
  delete[] nsim;
  delete[] nsimdone;

  return dat;
};




class distortion_formatting_error: public std::exception{
  static const char*const header;
  char * message;
public:
  distortion_formatting_error(const char *const mess){message=new char[strlen(mess)+1];strcpy(message,mess);};
  distortion_formatting_error(const char *const mess,const char*values,const char *end){message=new char[strlen(mess)+strlen(values)+strlen(end)+1];strcpy(message,mess);strcat(message,values);strcat(message,end);};
  virtual const char* what() const throw() {char *full_message=new char[strlen(header)+strlen(message)+1];strcpy(full_message,header);strcat(full_message,message);return full_message;}; 
};

const char*const distortion_formatting_error::header="Distortion formatting error: ";



Distortion* Read_distortion(unsigned int &argno,unsigned int argc,char *argv[]){
  if(argno>=argc){
    throw distortion_formatting_error("No Distortion specified.\n");
    return NULL;
  };
  char *distorttype=argv[argno++];
  if(!strcmp(distorttype,"contaminate")){
    if(argno>=argc){
      throw distortion_formatting_error("No contaminating probability or distribution specified.\n");
      return NULL;
    };
    long double contamprob=atof(argv[argno++]);
    if(argno>=argc){
      throw distortion_formatting_error("No contaminating distribution specified.\n");
      return NULL;
    };
    Distribution *contamDist=read_distribution(argc,argv,argno);
    if(contamDist==NULL){
      //This shouldn't happen as read_distribution should throw the exception
      throw distortion_formatting_error("Problem reading contaminating distribution.\n");
      return NULL;
    };
    return new contamination(contamprob,contamDist);
  }else if(!strcmp(distorttype,"other")){
    //User specified simulation distribution.
    //Treated as 100% contamination.
    if(argno>=argc){
      throw distortion_formatting_error("No Alternative distribution specified for distortion type \"other\".\n");
    };
    long double contamprob=1;
    Distribution *contamDist=read_distribution(argc,argv,argno);
    return new contamination(contamprob,contamDist);
  }else if(!strcmp(distorttype,"no_distortion")){
    return new no_distortion();
  }else if(!strcmp(distorttype,"sample_bias")){
    if(argno>=argc){
      throw distortion_formatting_error("No sample bias scheme specified.\n");
    };    
    char *sbtype=argv[argno++];
    if(!strcmp(sbtype,"step")){
      if(argno>=argc){
	throw distortion_formatting_error("No sample bias steps specified.\n");
      };        
      char *cuts=argv[argno++];  
      if(argno>=argc){
	throw distortion_formatting_error("No sample bias probabilities specified.\n");
      };        
      char *cutprobs=argv[argno++];  
      unsigned int nc=1;
      unsigned int ncp=1;
      for(unsigned int i=0;*(cuts+i);i++){
	if(*(cuts+i)==','){
	  nc++;
	};
      };
      for(unsigned int i=0;*(cutprobs+i);i++){
	if(*(cutprobs+i)==','){
	  ncp++;
	};
      };
      if(ncp!=nc+1){
	char *message=new char[75];//should be enough.
	sprintf(message,"Number of cuts=%d, Number of probabilities=%d",nc,ncp);
	throw distortion_formatting_error("for sample bias type step, Number of intervals and number of probabilities should match \n",message,"\n\t\tNote: Adequate bootstrap has no localisation - you must use \'.\' as decimal separator and \',\' to separate arguments.\n");
      };
      
      long double *sample_bias_step_cuts=new long double[nc];
      long double *sample_bias_step_cum_probs=new long double[nc+1];

      *sample_bias_step_cuts=atof(cuts);
      nc=1;
      for(unsigned int i=0;*(cuts+i+1);i++){
	if(*(cuts+i)==','){
	  *(sample_bias_step_cuts+nc++)=atof(cuts+i+1);
	};
      };
      ncp=1;
      *(sample_bias_step_cum_probs)=atof(cutprobs);
      long double totprob=*(sample_bias_step_cum_probs);
      for(unsigned int i=0;*(cutprobs+i+1);i++){
	if(*(cutprobs+i)==','){
	  totprob+=atof(cutprobs+i+1);
	  *(sample_bias_step_cum_probs+ncp++)=atof(cutprobs+i+1);
	};
      };      
      long double cumprob=*(sample_bias_step_cum_probs);      
      *(sample_bias_step_cum_probs)=cumprob/totprob;
      for(unsigned int j=1;j<=nc;j++){
	cumprob+=*(sample_bias_step_cum_probs+j);      
	*(sample_bias_step_cum_probs+j)=cumprob/totprob;	
      };
      return new sample_bias_step(nc,sample_bias_step_cuts,sample_bias_step_cum_probs);
    }else{
      throw distortion_formatting_error("Sample bias method \"",sbtype,"\" not currently supported.\n");
    };
  }else{
    throw distortion_formatting_error("Distortion type \"",distorttype,"\" not currently supported.\n");;
  };  
};

